@@ -1,12 +1,12 @@
-#!/usr/bin/python
-# Copyright (c) 2003-2016 CORE Security Technologies
+#!/usr/bin/env python
+# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
 # A similar approach to psexec w/o using RemComSvc. The technique is described here
-# http://www.accuvant.com/blog/owning-computers-without-shell-access
+# https://www.optiv.com/blog/owning-computers-without-shell-access
 # Our implementation goes one step further, instantiating a local smbserver to receive the 
 # output of the commands. This is useful in the situation where the target machine does NOT
 # have a writeable share available.
@@ -27,18 +27,22 @@
 #
 # Reference for:
 #  DCE/RPC and SMB.
-
+from __future__ import division
+from __future__ import print_function
 import sys
 import os
 import cmd
 import argparse
-import ConfigParser
+try:
+    import ConfigParser
+except ImportError:
+    import configparser as ConfigParser
 import logging
 from threading import Thread
 
 from impacket.examples import logger
 from impacket import version, smbserver
-from impacket.smbconnection import *
+from impacket.smbconnection import SMB_DIALECT
 from impacket.dcerpc.v5 import transport, scmr
 
 OUTPUT_FILENAME = '__output'
@@ -55,7 +59,7 @@ class SMBServer(Thread):
         logging.info('Cleaning up..')
         try:
             os.unlink(SMBSERVER_DIR + '/smb.log')
-        except:
+        except OSError:
             pass
         os.rmdir(SMBSERVER_DIR)
 
@@ -87,7 +91,7 @@ class SMBServer(Thread):
         logging.info('Creating tmp directory')
         try:
             os.mkdir(SMBSERVER_DIR)
-        except Exception, e:
+        except Exception as e:
             logging.critical(str(e))
             pass
         logging.info('Setting up SMB Server')
@@ -105,20 +109,12 @@ class SMBServer(Thread):
         self._Thread__stop()
 
 class CMDEXEC:
-    KNOWN_PROTOCOLS = {
-        '139/SMB': (r'ncacn_np:%s[\pipe\svcctl]', 139),
-        '445/SMB': (r'ncacn_np:%s[\pipe\svcctl]', 445),
-        }
-
-    def __init__(self, protocols=None,
-                 username='', password='', domain='', hashes=None, aesKey=None, doKerberos=None, kdcHost=None,
-                 mode=None, share=None):
-        if not protocols:
-            protocols = CMDEXEC.KNOWN_PROTOCOLS.keys()
+    def __init__(self, username='', password='', domain='', hashes=None, aesKey=None,
+                 doKerberos=None, kdcHost=None, mode=None, share=None, port=445):
 
         self.__username = username
         self.__password = password
-        self.__protocols = [protocols]
+        self.__port = port
         self.__serviceName = 'BTOBTO'
         self.__domain = domain
         self.__lmhash = ''
@@ -132,45 +128,39 @@ class CMDEXEC:
         if hashes is not None:
             self.__lmhash, self.__nthash = hashes.split(':')
 
-    def run(self, addr):
-        for protocol in self.__protocols:
-            protodef = CMDEXEC.KNOWN_PROTOCOLS[protocol]
-            port = protodef[1]
+    def run(self, remoteName, remoteHost):
+        stringbinding = r'ncacn_np:%s[\pipe\svcctl]' % remoteName
+        logging.debug('StringBinding %s'%stringbinding)
+        rpctransport = transport.DCERPCTransportFactory(stringbinding)
+        rpctransport.set_dport(self.__port)
+        rpctransport.setRemoteHost(remoteHost)
+        if hasattr(rpctransport,'preferred_dialect'):
+            rpctransport.preferred_dialect(SMB_DIALECT)
+        if hasattr(rpctransport, 'set_credentials'):
+            # This method exists only for selected protocol sequences.
+            rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash,
+                                         self.__nthash, self.__aesKey)
+        rpctransport.set_kerberos(self.__doKerberos, self.__kdcHost)
 
-            logging.info("Trying protocol %s..." % protocol)
-            logging.info("Creating service %s..." % self.__serviceName)
-
-            stringbinding = protodef[0] % addr
-
-            rpctransport = transport.DCERPCTransportFactory(stringbinding)
-            rpctransport.set_dport(port)
-
-            if hasattr(rpctransport,'preferred_dialect'):
-               rpctransport.preferred_dialect(SMB_DIALECT)
-            if hasattr(rpctransport, 'set_credentials'):
-                # This method exists only for selected protocol sequences.
-                rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash,
-                                             self.__nthash, self.__aesKey)
-            rpctransport.set_kerberos(self.__doKerberos, self.__kdcHost)
-
-            self.shell = None
-            try:
-                if self.__mode == 'SERVER':
-                    serverThread = SMBServer()
-                    serverThread.daemon = True
-                    serverThread.start()
-                self.shell = RemoteShell(self.__share, rpctransport, self.__mode, self.__serviceName)
-                self.shell.cmdloop()
-                if self.__mode == 'SERVER':
-                    serverThread.stop()
-            except  (Exception, KeyboardInterrupt), e:
-                #import traceback
-                #traceback.print_exc()
-                logging.critical(str(e))
-                if self.shell is not None:
-                    self.shell.finish()
-                sys.stdout.flush()
-                sys.exit(1)
+        self.shell = None
+        try:
+            if self.__mode == 'SERVER':
+                serverThread = SMBServer()
+                serverThread.daemon = True
+                serverThread.start()
+            self.shell = RemoteShell(self.__share, rpctransport, self.__mode, self.__serviceName)
+            self.shell.cmdloop()
+            if self.__mode == 'SERVER':
+                serverThread.stop()
+        except  (Exception, KeyboardInterrupt) as e:
+            if logging.getLogger().level == logging.DEBUG:
+                import traceback
+                traceback.print_exc()
+            logging.critical(str(e))
+            if self.shell is not None:
+                self.shell.finish()
+            sys.stdout.flush()
+            sys.exit(1)
 
 class RemoteShell(cmd.Cmd):
     def __init__(self, share, rpc, mode, serviceName):
@@ -179,7 +169,7 @@ class RemoteShell(cmd.Cmd):
         self.__mode = mode
         self.__output = '\\\\127.0.0.1\\' + self.__share + '\\' + OUTPUT_FILENAME
         self.__batchFile = '%TEMP%\\' + BATCH_FILENAME 
-        self.__outputBuffer = ''
+        self.__outputBuffer = b''
         self.__command = ''
         self.__shell = '%COMSPEC% /Q /c '
         self.__serviceName = serviceName
@@ -189,7 +179,7 @@ class RemoteShell(cmd.Cmd):
         self.__scmr = rpc.get_dce_rpc()
         try:
             self.__scmr.connect()
-        except Exception, e:
+        except Exception as e:
             logging.critical(str(e))
             sys.exit(1)
 
@@ -220,7 +210,7 @@ class RemoteShell(cmd.Cmd):
            scmr.hRDeleteService(self.__scmr, service)
            scmr.hRControlService(self.__scmr, service, scmr.SERVICE_CONTROL_STOP)
            scmr.hRCloseServiceHandle(self.__scmr, service)
-        except:
+        except scmr.DCERPCException:
            pass
 
     def do_shell(self, s):
@@ -233,15 +223,15 @@ class RemoteShell(cmd.Cmd):
         return False
 
     def do_cd(self, s):
-        # We just can't CD or mantain track of the target dir.
+        # We just can't CD or maintain track of the target dir.
         if len(s) > 0:
             logging.error("You can't CD under SMBEXEC. Use full paths.")
 
         self.execute_remote('cd ' )
         if len(self.__outputBuffer) > 0:
             # Stripping CR/LF
-            self.prompt = string.replace(self.__outputBuffer,'\r\n','') + '>'
-            self.__outputBuffer = ''
+            self.prompt = self.__outputBuffer.decode().replace('\r\n','') + '>'
+            self.__outputBuffer = b''
 
     def do_CD(self, s):
         return self.do_cd(s)
@@ -271,7 +261,8 @@ class RemoteShell(cmd.Cmd):
         command += ' & ' + 'del ' + self.__batchFile 
 
         logging.debug('Executing %s' % command)
-        resp = scmr.hRCreateServiceW(self.__scmr, self.__scHandle, self.__serviceName, self.__serviceName, lpBinaryPathName=command)
+        resp = scmr.hRCreateServiceW(self.__scmr, self.__scHandle, self.__serviceName, self.__serviceName,
+                                     lpBinaryPathName=command, dwStartType=scmr.SERVICE_DEMAND_START)
         service = resp['lpServiceHandle']
 
         try:
@@ -284,32 +275,44 @@ class RemoteShell(cmd.Cmd):
 
     def send_data(self, data):
         self.execute_remote(data)
-        print self.__outputBuffer
-        self.__outputBuffer = ''
+        print(self.__outputBuffer.decode())
+        self.__outputBuffer = b''
 
 
 # Process command-line arguments.
 if __name__ == '__main__':
     # Init the example's logger theme
     logger.init()
-    print version.BANNER
+    print(version.BANNER)
 
     parser = argparse.ArgumentParser()
 
     parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
-    parser.add_argument('-share', action='store', default = 'C$', help='share where the output will be grabbed from (default C$)')
-    parser.add_argument('-mode', action='store', choices = {'SERVER','SHARE'}, default='SHARE', help='mode to use (default SHARE, SERVER needs root!)')
+    parser.add_argument('-share', action='store', default = 'C$', help='share where the output will be grabbed from '
+                                                                       '(default C$)')
+    parser.add_argument('-mode', action='store', choices = {'SERVER','SHARE'}, default='SHARE',
+                        help='mode to use (default SHARE, SERVER needs root!)')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
 
-    parser.add_argument('protocol', choices=CMDEXEC.KNOWN_PROTOCOLS.keys(), nargs='?', default='445/SMB', help='transport protocol (default 445/SMB)')
+    group = parser.add_argument_group('connection')
+
+    group.add_argument('-dc-ip', action='store',metavar = "ip address", help='IP Address of the domain controller. '
+                       'If omitted it will use the domain part (FQDN) specified in the target parameter')
+    group.add_argument('-target-ip', action='store', metavar="ip address", help='IP Address of the target machine. If '
+                       'ommited it will use whatever was specified as target. This is useful when target is the NetBIOS '
+                       'name and you cannot resolve it')
+    group.add_argument('-port', choices=['139', '445'], nargs='?', default='445', metavar="destination port",
+                       help='Destination port to connect to SMB Server')
 
     group = parser.add_argument_group('authentication')
 
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
-    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
-    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
-    group.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller. If ommited it use the domain part (FQDN) specified in the target parameter')
+    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file '
+                       '(KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the '
+                       'ones specified in the command line')
+    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication '
+                                                                            '(128 or 256 bits)')
 
  
     if len(sys.argv)==1:
@@ -324,12 +327,12 @@ if __name__ == '__main__':
         logging.getLogger().setLevel(logging.INFO)
 
     import re
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
+    domain, username, password, remoteName = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
 
     #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    if '@' in remoteName:
+        password = password + '@' + remoteName.rpartition('@')[0]
+        remoteName = remoteName.rpartition('@')[2]
 
     if domain is None:
         domain = ''
@@ -338,13 +341,19 @@ if __name__ == '__main__':
         from getpass import getpass
         password = getpass("Password:")
 
+    if options.target_ip is None:
+        options.target_ip = remoteName
+
     if options.aesKey is not None:
         options.k = True
 
     try:
-        executer = CMDEXEC(options.protocol, username, password, domain, options.hashes, options.aesKey, options.k,
-                           options.dc_ip, options.mode, options.share)
-        executer.run(address)
-    except Exception, e:
+        executer = CMDEXEC(username, password, domain, options.hashes, options.aesKey, options.k,
+                           options.dc_ip, options.mode, options.share, int(options.port))
+        executer.run(remoteName, options.target_ip)
+    except Exception as e:
+        if logging.getLogger().level == logging.DEBUG:
+            import traceback
+            traceback.print_exc()
         logging.critical(str(e))
     sys.exit(0)
