@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2016 CORE Security Technologies
+# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -11,20 +11,22 @@
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://github.com/CoreSecurity/impacket/tree/master/impacket/testcases/SMB_RPC
+#   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too. 
 #
+
+
 from struct import unpack, pack
 
 from impacket.dcerpc.v5.ndr import NDRCALL, NDRSTRUCT, NDRPOINTER, NDRUniConformantVaryingArray, NDRUniConformantArray
 from impacket.dcerpc.v5.dtypes import DWORD, UUID, ULONG, LPULONG, BOOLEAN, SECURITY_INFORMATION, PFILETIME, \
     RPC_UNICODE_STRING, FILETIME, NULL, MAXIMUM_ALLOWED, OWNER_SECURITY_INFORMATION, PWCHAR, PRPC_UNICODE_STRING
 from impacket.dcerpc.v5.rpcrt import DCERPCException
-from impacket import system_errors
+from impacket import system_errors, LOG
 from impacket.uuid import uuidtup_to_bin
 
 MSRPC_UUID_RRP = uuidtup_to_bin(('338CD001-2244-31F1-AAAA-900038001003', '1.0'))
@@ -35,7 +37,7 @@ class DCERPCSessionError(DCERPCException):
 
     def __str__( self ):
         key = self.error_code
-        if system_errors.ERROR_MESSAGES.has_key(key):
+        if key in system_errors.ERROR_MESSAGES:
             error_msg_short = system_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = system_errors.ERROR_MESSAGES[key][1] 
             return 'RRP SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
@@ -709,7 +711,7 @@ def packValue(valueType, value):
 
 def unpackValue(valueType, value):
     if valueType == REG_DWORD:
-        retData = unpack('<L', ''.join(value))[0]
+        retData = unpack('<L', b''.join(value))[0]
     elif valueType == REG_DWORD_BIG_ENDIAN:
         retData = unpack('>L', ''.join(value))[0]
     elif valueType == REG_EXPAND_SZ:
@@ -721,9 +723,9 @@ def unpackValue(valueType, value):
     elif valueType == REG_QWORD_LITTLE_ENDIAN:
         retData = unpack('>Q', ''.join(value))[0]
     elif valueType == REG_SZ:
-        retData = ''.join(value).decode('utf-16le')
+        retData = b''.join(value).decode('utf-16le')
     else:
-        retData = ''.join(value)
+        retData = b''.join(value)
 
     return retData
 
@@ -788,28 +790,38 @@ def hBaseRegEnumKey(dce, hKey, dwIndex, lpftLastWriteTime = NULL):
     request['hKey'] = hKey
     request['dwIndex'] = dwIndex
     request.fields['lpNameIn'].fields['MaximumLength'] = 1024
-    request.fields['lpNameIn'].fields['Data'].fields['Data'].fields['MaximumCount'] = 1024/2
+    request.fields['lpNameIn'].fields['Data'].fields['Data'].fields['MaximumCount'] = 1024//2
     request['lpClassIn'] = ' '* 64
     request['lpftLastWriteTime'] = lpftLastWriteTime
 
     return dce.request(request)
 
-def hBaseRegEnumValue(dce, hKey, dwIndex, dataLen=128):
+def hBaseRegEnumValue(dce, hKey, dwIndex, dataLen=256):
     request = BaseRegEnumValue()
     request['hKey'] = hKey
     request['dwIndex'] = dwIndex
+    retries = 1
 
     # We need to be aware the size might not be enough, so let's catch ERROR_MORE_DATA exception
     while True:
         try:
-            request['lpValueNameIn'] = ' ' * dataLen
-            request['lpData'] = ' ' * dataLen
+            # Only the maximum length field of the lpValueNameIn is used to determine the buffer length to be allocated
+            # by the service. Specify a string with a zero length but maximum length set to the largest buffer size
+            # needed to hold the value names.
+            request.fields['lpValueNameIn'].fields['MaximumLength'] = dataLen*2
+            request.fields['lpValueNameIn'].fields['Data'].fields['Data'].fields['MaximumCount'] = dataLen
+
+            request['lpData'] = b' ' * dataLen
             request['lpcbData'] = dataLen
             request['lpcbLen'] = dataLen
             resp = dce.request(request)
-        except DCERPCSessionError, e:
+        except DCERPCSessionError as e:
+            if retries > 1:
+                LOG.debug('Too many retries when calling hBaseRegEnumValue, aborting')
+                raise
             if e.get_error_code() == system_errors.ERROR_MORE_DATA:
                 # We need to adjust the size
+                retries +=1
                 dataLen = e.get_packet()['lpcbData']
                 continue
             else:
@@ -860,22 +872,26 @@ def hBaseRegQueryInfoKey(dce, hKey):
     # Not the cleanest way, but oh well
     # Plus, Windows XP needs MaximumCount also set
     request.fields['lpClassIn'].fields['MaximumLength'] = 1024
-    request.fields['lpClassIn'].fields['Data'].fields['Data'].fields['MaximumCount'] = 1024/2
+    request.fields['lpClassIn'].fields['Data'].fields['Data'].fields['MaximumCount'] = 1024//2
     return dce.request(request)
 
 def hBaseRegQueryValue(dce, hKey, lpValueName, dataLen=512):
     request = BaseRegQueryValue()
     request['hKey'] = hKey
     request['lpValueName'] = checkNullString(lpValueName)
+    retries = 1
 
     # We need to be aware the size might not be enough, so let's catch ERROR_MORE_DATA exception
     while True:
         try:
-            request['lpData'] = ' ' * dataLen
+            request['lpData'] =b' ' * dataLen
             request['lpcbData'] = dataLen
             request['lpcbLen'] = dataLen
             resp = dce.request(request)
-        except DCERPCSessionError, e:
+        except DCERPCSessionError as e:
+            if retries > 1:
+                LOG.debug('Too many retries when calling hBaseRegQueryValue, aborting')
+                raise
             if e.get_error_code() == system_errors.ERROR_MORE_DATA:
                 # We need to adjust the size
                 dataLen = e.get_packet()['lpcbData']
@@ -946,7 +962,7 @@ def hBaseRegQueryMultipleValues(dce, hKey, val_listIn):
         request['val_listIn'].append(itemn)
 
     request['num_vals'] = len(request['val_listIn'])
-    request['lpvalueBuf'] = list(' '*128)
+    request['lpvalueBuf'] = list(b' '*128)
     request['ldwTotsize'] = 128
 
     resp = dce.request(request)
@@ -984,4 +1000,3 @@ def hBaseRegDeleteValue(dce, hKey, lpValueName):
     request['hKey'] = hKey
     request['lpValueName'] = checkNullString(lpValueName)
     return dce.request(request)
-
