@@ -1,26 +1,32 @@
-# Copyright (c) 2003-2014 CORE Security Technologies
+# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   [MS-SAMR] Interface implementation
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://github.com/CoreSecurity/impacket/tree/master/impacket/testcases/SMB_RPC
+#   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too. 
 #
-from impacket.dcerpc.v5 import ndr
-from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRUNION, NDRPOINTER, NDRUniConformantArray, NDRUniConformantVaryingArray, NDRENUM, NULL
-from impacket.dcerpc.v5.dtypes import *
+from __future__ import division
+from __future__ import print_function
+from binascii import unhexlify
+
+from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRUNION, NDRPOINTER, NDRUniConformantArray, \
+    NDRUniConformantVaryingArray, NDRENUM
+from impacket.dcerpc.v5.dtypes import NULL, RPC_UNICODE_STRING, ULONG, USHORT, UCHAR, LARGE_INTEGER, RPC_SID, LONG, STR, \
+    LPBYTE, SECURITY_INFORMATION, PRPC_SID, PRPC_UNICODE_STRING, LPWSTR
+from impacket.dcerpc.v5.rpcrt import DCERPCException
 from impacket import nt_errors, LOG
 from impacket.uuid import uuidtup_to_bin
 from impacket.dcerpc.v5.enum import Enum
@@ -28,29 +34,18 @@ from impacket.structure import Structure
 
 MSRPC_UUID_SAMR   = uuidtup_to_bin(('12345778-1234-ABCD-EF00-0123456789AC', '1.0'))
 
-class DCERPCSessionError(Exception):
-    def __init__( self, packet = None, error_code = None):
-        Exception.__init__(self)
-        self.packet = packet
-        if packet is not None:
-            self.error_code = packet['ErrorCode']
-        else:
-            self.error_code = error_code
-       
-    def get_error_code( self ):
-        return self.error_code
- 
-    def get_packet( self ):
-        return self.packet
+class DCERPCSessionError(DCERPCException):
+    def __init__(self, error_string=None, error_code=None, packet=None):
+        DCERPCException.__init__(self, error_string, error_code, packet)
 
     def __str__( self ):
         key = self.error_code
-        if (nt_errors.ERROR_MESSAGES.has_key(key)):
+        if key in nt_errors.ERROR_MESSAGES:
             error_msg_short = nt_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = nt_errors.ERROR_MESSAGES[key][1] 
             return 'SAMR SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
         else:
-            return 'SAMR SessionError: unknown error code: 0x%x' % (self.error_code)
+            return 'SAMR SessionError: unknown error code: 0x%x' % self.error_code
 
 ################################################################################
 # CONSTANTS
@@ -250,6 +245,7 @@ UF_USE_AES_KEYS                           = 0x08000000
 DOMAIN_USER_RID_ADMIN                 = 0x000001F4
 DOMAIN_USER_RID_GUEST                 = 0x000001F5
 DOMAIN_USER_RID_KRBTGT                = 0x000001F6
+DOMAIN_GROUP_RID_ADMINS               = 0x00000200
 DOMAIN_GROUP_RID_USERS                = 0x00000201
 DOMAIN_GROUP_RID_COMPUTERS            = 0x00000203
 DOMAIN_GROUP_RID_CONTROLLERS          = 0x00000204
@@ -304,12 +300,12 @@ class RPC_STRING(NDRSTRUCT):
     )
 
     def dump(self, msg = None, indent = 0):
-        if msg is None: msg = self.__class__.__name__
-        ind = ' '*indent
+        if msg is None:
+            msg = self.__class__.__name__
         if msg != '':
-            print "%s" % (msg),
+            print("%s" % msg, end=' ')
         # Here just print the data
-        print " %r" % (self['Data']),
+        print(" %r" % (self['Data']), end=' ')
 
 class PRPC_STRING(NDRPOINTER):
     referent = (
@@ -355,15 +351,20 @@ class RPC_SHORT_BLOB(NDRSTRUCT):
     )
 
 # 2.2.3.2 SAMPR_HANDLE
-class SAMPR_HANDLE(NDR):
+class SAMPR_HANDLE(NDRSTRUCT):
     structure =  (
-        ('Data','20s=""'),
+        ('Data','20s=b""'),
     )
+    def getAlignment(self):
+        if self._isNDR64 is True:
+            return 8
+        else:
+            return 4
 
 # 2.2.3.3 ENCRYPTED_LM_OWF_PASSWORD, ENCRYPTED_NT_OWF_PASSWORD
 class ENCRYPTED_LM_OWF_PASSWORD(NDRSTRUCT):
     structure = (
-        ('Data', '16s=""'),
+        ('Data', '16s=b""'),
     )
     def getAlignment(self):
         return 1
@@ -992,7 +993,7 @@ class SAMPR_USER_LOGON_HOURS_INFORMATION(NDRSTRUCT):
 # 2.2.7.21 SAMPR_ENCRYPTED_USER_PASSWORD
 class SAMPR_USER_PASSWORD(NDRSTRUCT):
     structure = (
-        ('Buffer', '512s=""'),
+        ('Buffer', '512s=b""'),
         ('Length', ULONG),
     )
     def getAlignment(self):
@@ -1001,7 +1002,7 @@ class SAMPR_USER_PASSWORD(NDRSTRUCT):
 
 class SAMPR_ENCRYPTED_USER_PASSWORD(NDRSTRUCT):
     structure = (
-        ('Buffer', '516s=""'),
+        ('Buffer', '516s=b""'),
     )
     def getAlignment(self):
         return 1
@@ -1014,7 +1015,7 @@ class PSAMPR_ENCRYPTED_USER_PASSWORD(NDRPOINTER):
 # 2.2.7.22 SAMPR_ENCRYPTED_USER_PASSWORD_NEW
 class SAMPR_ENCRYPTED_USER_PASSWORD_NEW(NDRSTRUCT):
     structure = (
-        ('Buffer', '522s=""'),
+        ('Buffer', '522s=b""'),
     )
     def getAlignment(self):
         return 1
@@ -1119,7 +1120,7 @@ class PSAMPR_USER_INFO_BUFFER(NDRPOINTER):
 
 class PSAMPR_SERVER_NAME2(NDRPOINTER):
     referent = (
-        ('Data', '4s=""'),
+        ('Data', '4s=b""'),
     ) 
 
 # 2.2.8.2 SAMPR_DOMAIN_DISPLAY_USER
@@ -2750,40 +2751,43 @@ def hSamrChangePasswordUser(dce, userHandle, oldPassword, newPassword):
 
     return dce.request(request)
 
-def hSamrUnicodeChangePasswordUser2(dce, serverName='\x00', userName='', oldPassword='', newPassword='', oldPwdHashLM = None, oldPwdHashNT = None):
+def hSamrUnicodeChangePasswordUser2(dce, serverName='\x00', userName='', oldPassword='', newPassword='', oldPwdHashLM = '', oldPwdHashNT = ''):
     request = SamrUnicodeChangePasswordUser2()
     request['ServerName'] = serverName
     request['UserName'] = userName
 
     try:
-        from Crypto.Cipher import ARC4
+        from Cryptodome.Cipher import ARC4
     except Exception:
-        LOG.critical("Warning: You don't have any crypto installed. You need PyCrypto")
-        LOG.critical("See http://www.pycrypto.org/")
+        LOG.critical("Warning: You don't have any crypto installed. You need pycryptodomex")
+        LOG.critical("See https://pypi.org/project/pycryptodomex/")
     from impacket import crypto, ntlm
 
-    if oldPwdHashLM is None and oldPwdHashNT is None:
+    if oldPwdHashLM == '' and oldPwdHashNT == '':
         oldPwdHashLM = ntlm.LMOWFv1(oldPassword)
         oldPwdHashNT = ntlm.NTOWFv1(oldPassword)
     else:
         # Let's convert the hashes to binary form, if not yet
         try:
-            oldPwdHashLM = oldPwdHashLM.decode('hex')
+            oldPwdHashLM = unhexlify(oldPwdHashLM)
         except:
             pass
         try: 
-            oldPwdHashNT = oldPwdHashNT.decode('hex')
+            oldPwdHashNT = unhexlify(oldPwdHashNT)
         except:
             pass
 
     newPwdHashNT = ntlm.NTOWFv1(newPassword)
-    newPwdHashLM = ntlm.LMOWFv1(newPassword)
-
 
     samUser = SAMPR_USER_PASSWORD()
-    samUser['Buffer'] = 'A'*(512-len(newPassword)*2) + newPassword.encode('utf-16le')
+    try:
+        samUser['Buffer'] = b'A'*(512-len(newPassword)*2) + newPassword.encode('utf-16le')
+    except UnicodeDecodeError:
+        import sys
+        samUser['Buffer'] = b'A'*(512-len(newPassword)*2) + newPassword.decode(sys.getfilesystemencoding()).encode('utf-16le')
+
     samUser['Length'] = len(newPassword)*2
-    pwdBuff = str(samUser)
+    pwdBuff = samUser.getData()
 
     rc4 = ARC4.new(oldPwdHashNT)
     encBuf = rc4.encrypt(pwdBuff)
@@ -2865,12 +2869,11 @@ def hSamrLookupIdsInDomain(dce, domainHandle, ids):
     request = SamrLookupIdsInDomain()
     request['DomainHandle'] =  domainHandle
     request['Count'] = len(ids)
-    for id in ids:
+    for dId in ids:
         entry = ULONG()
-        entry['Data'] = id
+        entry['Data'] = dId
         request['RelativeIds'].append(entry)
 
     request.fields['RelativeIds'].fields['MaximumCount'] = 1000
 
     return dce.request(request)
-
