@@ -1,16 +1,16 @@
-# Copyright (c) 2003-2012 CORE Security Technologies)
+# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: smb3structs.py 689 2012-08-20 18:23:35Z bethus@gmail.com $
-#
-# Author: Alberto Solino (beto@coresecurity.com)
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   SMB 2 and 3 Protocol Structures and constants [MS-SMB2]
 #
+from __future__ import division
+from __future__ import print_function
 
 from impacket.structure import Structure
 
@@ -70,6 +70,8 @@ SMB2_GLOBAL_CAP_ENCRYPTION         = 0x40
 SMB2_DIALECT_002      = 0x0202 
 SMB2_DIALECT_21       = 0x0210 
 SMB2_DIALECT_30       = 0x0300 
+SMB2_DIALECT_302      = 0x0302  #SMB 3.0.2
+SMB2_DIALECT_311      = 0x0311  #SMB 3.1.1
 SMB2_DIALECT_WILDCARD = 0x02FF 
 
 # SMB2_SESSION_SETUP
@@ -273,6 +275,7 @@ FSCTL_SRV_COPYCHUNK_WRITE            = 0x001480F2
 FSCTL_LMR_REQUEST_RESILIENCY         = 0x001401D4
 FSCTL_QUERY_NETWORK_INTERFACE_INFO   = 0x001401FC
 FSCTL_SET_REPARSE_POINT              = 0x000900A4
+FSCTL_DELETE_REPARSE_POINT           = 0x000900AC
 FSCTL_DFS_GET_REFERRALS_EX           = 0x000601B0
 FSCTL_FILE_LEVEL_TRIM                = 0x00098208
 FSCTL_VALIDATE_NEGOTIATE_INFO        = 0x00140204
@@ -346,6 +349,7 @@ SMB2_0_INFO_SECURITY    = 0x03
 SMB2_0_INFO_QUOTA       = 0x04
 
 # File Information Classes
+SMB2_SEC_INFO_00                      = 0
 SMB2_FILE_ACCESS_INFO                 = 8
 SMB2_FILE_ALIGNMENT_INFO              = 17
 SMB2_FILE_ALL_INFO                    = 18
@@ -391,7 +395,17 @@ SMB2_FILE_STREAM_INFO                 = 22
 SMB2_FILE_TRACKING_INFO               = 36
 SMB2_FILE_VALID_DATA_LENGTH_INFO      = 39
 
-
+# File System Information Classes
+SMB2_FILESYSTEM_VOLUME_INFO           = 1
+SMB2_FILESYSTEM_LABEL_INFO            = 2
+SMB2_FILESYSTEM_SIZE_INFO             = 3
+SMB2_FILESYSTEM_DEVICE_INFO           = 4
+SMB2_FILESYSTEM_ATTRIBUTE_INFO        = 5
+SMB2_FILESYSTEM_CONTROL_INFO          = 6
+SMB2_FILESYSTEM_FULL_SIZE_INFO        = 7
+SMB2_FILESYSTEM_OBJECT_ID_INFO        = 8
+SMB2_FILESYSTEM_DRIVER_PATH_INFO      = 9
+SMB2_FILESYSTEM_SECTOR_SIZE_INFO      = 11
 
 # Additional information
 OWNER_SECURITY_INFORMATION  = 0x00000001
@@ -407,6 +421,7 @@ SL_INDEX_SPECIFIED      = 0x00000004
 
 # TRANSFORM_HEADER
 SMB2_ENCRYPTION_AES128_CCM = 0x0001
+SMB2_ENCRYPTION_AES128_GCM = 0x0002
 
 
 # STRUCtures
@@ -414,11 +429,11 @@ SMB2_ENCRYPTION_AES128_CCM = 0x0001
 class SMBPacketBase(Structure):
     def addCommand(self,command):
         # Pad to 8 bytes and put the offset of another SMBPacket
-        raise 'Implement This!' 
+        raise Exception('Implement This!')
 
     def isValidAnswer(self, status):
         if self['Status'] != status:
-            import smb3
+            from . import smb3
             raise smb3.SessionError(self['Status'], self)
         return True
 
@@ -502,11 +517,11 @@ class SMB3Packet(SMBPacketBase):
 
 class SMB2Error(Structure):
     structure = (
-        ('StructureSize','<H=0'),
+        ('StructureSize','<H=9'),
         ('Reserved','<H=0'),
         ('ByteCount','<L=0'),
         ('_ErrorData','_-ErrorData','self["ByteCount"]'),
-        ('ErrorData',':'),
+        ('ErrorData','"\xff'),
     )
 
 class SMB2ErrorSymbolicLink(Structure):
@@ -890,7 +905,7 @@ class SMB2Read(Structure):
         ('_AlignPad','_-AlignPad','self["ReadChannelInfoOffset"] - (64 + self["StructureSize"] - 1)'),
         ('AlignPad',':=""'),
         ('_Buffer','_-Buffer','self["ReadChannelInfoLength"]'),
-        ('Buffer',':=0'),
+        ('Buffer',':="0"'),
     )
     def __init__(self, data = None):
         Structure.__init__(self,data)
@@ -985,16 +1000,6 @@ class SMB2LeaseBreakAcknowledgement(Structure):
 SMB2LeaseBreakResponse = SMB2LeaseBreakAcknowledgement
 
 # SMB2_LOCK
-class SMB2Lock(Structure):
-    structure = (
-        ('StructureSize','<H=48'),
-        ('LockCount','<H=0'),
-        ('LockSequence','<L=0'),
-        ('FileID',':',SMB2_FILEID),
-        ('_Locks','_-Locks','self["LockCount"]*len(SMB2_LOCK_ELEMENT)'),
-        ('Locks',':'),
-    )
-
 class SMB2_LOCK_ELEMENT(Structure):
     structure = (
         ('Offset','<Q=0'),
@@ -1003,6 +1008,16 @@ class SMB2_LOCK_ELEMENT(Structure):
         ('Reserved','<L=0'),
     )
 
+class SMB2Lock(Structure):
+    structure = (
+        ('StructureSize','<H=48'),
+        ('LockCount','<H=0'),
+        ('LockSequence','<L=0'),
+        ('FileID',':',SMB2_FILEID),
+        ('_Locks','_-Locks','self["LockCount"]*24'),
+        ('Locks',':'),
+    )
+
 class SMB2Lock_Response(Structure):
     structure = (
         ('StructureSize','<H=4'),
@@ -1042,8 +1057,8 @@ class SMB2Ioctl(Structure):
         ('MaxOutputResponse','<L=0'),
         ('Flags','<L=0'),
         ('Reserved2','<L=0'),
-        ('_AlignPad','_-AlignPad','self["InputOffset"] + self["OutputOffset"] - (64 + self["StructureSize"] - 1)'),
-        ('AlignPad',':=""'),
+        #('_AlignPad','_-AlignPad','self["InputOffset"] + self["OutputOffset"] - (64 + self["StructureSize"] - 1)'),
+        #('AlignPad',':=""'),
         ('_Buffer','_-Buffer','self["InputCount"]+self["OutputCount"]'),
         ('Buffer',':'),
     )
@@ -1106,8 +1121,16 @@ class VALIDATE_NEGOTIATE_INFO(Structure):
         ('Capabilities','<L=0'),
         ('Guid','16s=""'),
         ('SecurityMode','<H=0'),
-        ('DialectCount','<H=0'),
-        ('Dialects','*<H'),
+        #('DialectCount','<H=0'),
+        ('Dialects','<H*<H'),
+    )
+
+class VALIDATE_NEGOTIATE_INFO_RESPONSE(Structure):
+    structure = (
+        ('Capabilities','<L=0'),
+        ('Guid','16s=""'),
+        ('SecurityMode','<H=0'),
+        ('Dialect','<H'),
     )
 
 class SRV_SNAPSHOT_ARRAY(Structure):
@@ -1170,12 +1193,25 @@ class NETWORK_INTERFACE_INFO(Structure):
         ('SockAddr_Storage','128s=""'),
     )
 
-class VALIDATE_NEGOTIATE_INFO(Structure):
+class MOUNT_POINT_REPARSE_DATA_STRUCTURE(Structure):
     structure = (
-        ('Capabilities','<L=0'),
-        ('Guid','16s=""'),
-        ('SecurityMode','<H=0'),
-        ('Dialect','<H=0'),
+        ("ReparseTag", "<L=0xA0000003"),
+        ("ReparseDataLen", "<H=len(self['PathBuffer']) + 8"),
+        ("Reserved", "<H=0"),
+        ("SubstituteNameOffset", "<H=0"),
+        ("SubstituteNameLength", "<H=0"),
+        ("PrintNameOffset", "<H=0"),
+        ("PrintNameLength", "<H=0"),
+        ("PathBuffer", ":")
+    )
+
+class MOUNT_POINT_REPARSE_GUID_DATA_STRUCTURE(Structure):
+    structure = (
+        ("ReparseTag", "<L=0xA0000003"),
+        ("ReparseDataLen", "<H=len(self['DataBuffer'])"),
+        ("Reserved", "<H=0"),
+        ("ReparseGuid", "16s=''"),
+        ("DataBuffer", ":")
     )
 
 class SMB2Ioctl_Response(Structure):
@@ -1339,7 +1375,7 @@ class FILE_RENAME_INFORMATION_TYPE_2(Structure):
     structure = (
         ('ReplaceIfExists','<B=0'),
         ('Reserved','7s=""'),
-        ('RootDirectory',':',SMB2_FILEID),
+        ('RootDirectory','<Q=0'),
         ('FileNameLength','<L=0'),
         ('_FileName','_-FileName','self["FileNameLength"]'),
         ('FileName',':'),
@@ -1356,4 +1392,19 @@ class SMB2_TRANSFORM_HEADER(Structure):
         ('SessionID','<Q=0'),
     )
 
+# SMB2_FILE_INTERNAL_INFO
+class FileInternalInformation(Structure):
+    structure = (
+        ('IndexNumber','<q=0'),
+    )
 
+# SMB2_SEC_INFO_00       
+class FileSecInformation(Structure):
+    structure = (
+        ('Revision','<h=1'),
+        ('Type','<h=0'),
+        ('OffsetToOwner','<I=0'),
+        ('OffsetToGroup','<I=0'),
+        ('OffsetToSACL','<I=0'),
+        ('OffsetToDACL','<I=0'),
+    )
